process.env.NODE_ENV = 'development';

var paths = require('../config/paths');
var path = require('path');
var mkdir = require('mkdirp');

mkdir(paths.appRoot + '/logs', (err) => err ? "Could not create logs folder" : "Logs at: /logs");

var setUpRoutes = require(path.join(paths.appRoot, 'setup')),
    chalk = require('chalk'),
    detect = require('detect-port'),
    setUpWebpackDevServer = require('../webpack/devserver');

var openBrowser = require('react-dev-utils/openBrowser');
var clearConsole = require('react-dev-utils/clearConsole');
var formatWebpackMessages = require('react-dev-utils/formatWebpackMessages');
var getProcessForPort = require('react-dev-utils/getProcessForPort');
var prompt = require('react-dev-utils/prompt');
var fs = require('fs');
var checkRequiredFiles = require('react-dev-utils/checkRequiredFiles');

var jsFiles = [], cssFiles = [];
var useYarn = fs.existsSync(paths.yarnLockFile);
var cli = useYarn ? 'yarn' : 'npm';
var isInteractive = process.stdout.isTTY;

// Warn and crash if required files are missing
if (!checkRequiredFiles([paths.appViews + '/index.pug', paths.appIndexJs])) {
    console.error(chalk.red(`Missing required files. check whether ${paths.appView}/index.pug or ${paths.appIndexJs}.js is missing.`));
    process.exit(1);
}

var devServer = setUpWebpackDevServer(setUpRoutes);

// START THE SERVER
var DEFAULT_PORT = process.env.PORT || 3000;
var protocol = process.env.HTTPS === 'true' ? "https" : "http";
var host = process.env.HOST || 'localhost';

function run(port) {
    devServer.listen(port, function (err) {
        if (err) {
            console.error(chalk.red('Server start failed: %s port: %s, env: %s, instance: %s, version: %s',
                err, port, app.get('env'), process.env.NODE_APP_INSTANCE, process.version));
        } else {
            console.log();
            console.log('The app is running at:');
            console.log();
            console.log('  ' + chalk.cyan(protocol + '://' + host + ':' + port + '/'));
            console.log();
            console.log('Note that the development build is not optimized.');
            console.log('To create a production build, use ' + chalk.cyan(cli + ' run build') + '.');
            console.log();
            openBrowser(protocol + '://' + host + ':' + port + '/');
        }
    });
}

// We attempt to use the default port but if it is busy, we offer the user to
// run on a different port. `detect()` Promise resolves to the next free port.
detect(DEFAULT_PORT).then(port => {
    if (port === DEFAULT_PORT) {
        run(port);
        return;
    }

    if (isInteractive) {
        clearConsole();
        var existingProcess = getProcessForPort(DEFAULT_PORT);
        var question =
            chalk.yellow('Something is already running on port ' + DEFAULT_PORT + '.' +
                ((existingProcess) ? ' Probably:\n  ' + existingProcess : '')) +
            '\n\nWould you like to run the app on another port instead?';

        prompt(question, true).then(shouldChangePort => {
            if (shouldChangePort) {
                run(port);
            }
        });
    } else {
        console.log(chalk.red('Something is already running on port ' + DEFAULT_PORT + '.'));
    }
});

process.on('SIGTERM', function (e) {
    console.error('Process termination request received. Stoping taking new requests.');
    server.close(function () {
        console.error('Finished all pending requests. Terminating now.');
        process.exit(0);
    });
});
process.on("SIGINT", function (e) {
    console.error(e);
    process.exit(0);
});
