var webpackDevMiddleware = require('webpack-dev-middleware');
var webpackConfiguration = require('./webpack.config.dev');
var webpack = require('webpack');
var clearConsole = require('react-dev-utils/clearConsole');
var isInteractive = process.stdout.isTTY;
var formatWebpackMessages = require('react-dev-utils/formatWebpackMessages');
var chalk = require('chalk');
var WebpackDevServer = require('webpack-dev-server');
var paths = require('../config/paths')

clearConsole();

var defaultResources = {
    'styles.css': 'styles.css',
    'vendors.js': 'vendors.js',
    'app.js': 'app.js'
}

module.exports = function(setUpRoutes) {

    var compiler = webpack(webpackConfiguration); 
    var isFirstCompile = true;

    compiler.plugin('invalid', function() {
        if (isInteractive)
            clearConsole();
        console.log('Compiling...');
    });

    compiler.plugin('done', function(stats) {
        // We have switched off the default Webpack output in WebpackDevServer
        // options so we are going to "massage" the warnings and errors and present
        // them in a readable focused way.
        var messages = formatWebpackMessages(stats.toJson({}, true));
        var isSuccessful = !messages.errors.length && !messages.warnings.length;
        var showInstructions = isSuccessful && (isInteractive || isFirstCompile);
        var time = stats.endTime - stats.startTime;

        if (!messages.errors.length && !messages.warnings.length ) {
            console.log(chalk.green('Compiled successfully in ' + time + ' ms.'));
        }
            
        // If errors exist, only show errors.
        if (messages.errors.length) {
            console.log(chalk.red('Failed to compile in ' + time +' ms.'));
            console.log();
            messages.errors.forEach(message => {
                console.log(message);
                console.log();
            });
            return;
        }

        // Show warnings if no errors were found.
        if (messages.warnings.length) {
            console.log(chalk.yellow('Compiled with warnings in ' + time + ' ms'));
            console.log();
            messages.warnings.forEach(message => {
                console.log(message);
                console.log();
            });

            // Teach some ESLint tricks.
            console.log('You may use special comments to disable some warnings.');
            console.log('Use ' + chalk.yellow('// eslint-disable-next-line') + ' to ignore the next line.');
            console.log('Use ' + chalk.yellow('/* eslint-disable */') + ' to ignore all warnings in a file.');
        }
    });

    var devServer = new WebpackDevServer(compiler, {
        compress: true,
        clientLogLevel: 'none',
        historyApiFallback: true,
        contentBase: paths.appPublic,
        watchContentBase: true,
        hot: true,
        publicPath: webpackConfiguration.output.publicPath,
        quiet: true,
        watchOptions: {
            ignored: /node_modules/
        },
        setup: function(app) {
            setTimeout(function(){
                setUpRoutes(app, defaultResources);
            }, 0);
        }
    });
    return devServer
}