var webpack = require('webpack');
var path = require('path');
var resolve = path.resolve;
var DuplicatePackageCheckerPlugin = require('duplicate-package-checker-webpack-plugin');
var ExtractTextPlugin = require("extract-text-webpack-plugin");
var CopyWebpackPlugin = require('copy-webpack-plugin');
var WatchMissingNodeModulesPlugin = require('react-dev-utils/WatchMissingNodeModulesPlugin');
var ManifestPlugin = require('webpack-manifest-plugin');
var fs = require('fs');

var paths = require('../config/paths')

let styleName = "styles.css";
let outputFile = '[name].js';

let plugins = [
    new WatchMissingNodeModulesPlugin(paths.appNodeModules),
    new DuplicatePackageCheckerPlugin(),
    new webpack.HotModuleReplacementPlugin(),
    new webpack.NamedModulesPlugin(),
    new ExtractTextPlugin(styleName),
    new webpack.DefinePlugin({
      'process.env': {
        'NODE_ENV': JSON.stringify(process.env.NODE_ENV)
      }
    }),
    new CopyWebpackPlugin([{from: resolve(paths.appNodeModules, "unity-theme/dist/fonts"), to: "fonts"}]),
    new CopyWebpackPlugin([{from: resolve(paths.appNodeModules, 'unity-theme/dist/img'), to: "img"}]),
    new CopyWebpackPlugin([{from: resolve(paths.appNodeModules, "unity-theme/dist/theme.dev.css")}]),
    new ManifestPlugin({
        fileName: 'asset-manifest.json'
    })
];

let appEntryPoint = [
  'react-dev-utils/webpackHotDevClient',
  resolve(__dirname, '../src/root')
];
var unityConfigBrowser = fs.existsSync(path.join(paths.appNodeModules, 'unity-config', 'browser.js'));
var vendors = ['react', 'react-dom']

if (fs.existsSync(path.join(paths.appNodeModules, 'react-router'))) {
  vendors.push('react-router')
}

if (fs.existsSync(path.join(paths.appNodeModules, 'react-router-dom'))) {
  vendors.push('react-router-dom')
}

var config = {
  resolve: {
    aliasFields: ["browser"],
    modules: ['node_modules'].concat(paths.nodePaths),
    extensions: [".js", ".jsx", ".json"],
    alias: {
      'ProjectSrc': paths.appSrc,
      'react': path.join(paths.appNodeModules, 'react'),
      'unity-context': 'unity-context/browser',
      'unity-config': unityConfigBrowser ? 'unity-config/browser' : 'unity-config'
    }
  },
  resolveLoader: {
    modules: [paths.appNodeModules, path.resolve(__dirname, "../node_modules/"), "node_modules"]
  },
  entry: {
    app: appEntryPoint,
    vendors: vendors
  },
  devtool: 'cheap-source-map',
  output: {
    path: paths.appDist,
    filename: outputFile,
    pathinfo: true,
    publicPath: '/'
  },
  module: {
    rules: [
      {
        test: /\.jade$/, 
        loader: 'jade-loader'
      },
      {
        test: /\.(js)$/,
        loader: 'eslint-loader',
        enforce: 'pre',
        include: paths.appSrc,
        options: {
            configFile: path.join(__dirname, '../.eslintrc'),
            useEslintrc: false
        }
      },
      {
        test: /\.jsx?$/,
        loader: 'babel-loader',
        include: [paths.appSrc, path.resolve(paths.appNodeModules, 'unity-components')],
        options: {
            cacheDirectory: true,
            babelrc: false,
            presets: [[require.resolve("babel-preset-es2015")], require.resolve("babel-preset-stage-0"), require.resolve("babel-preset-react")]
        },
      },
      {
          test: /\.(png|jpg|jpeg|gif|svg|woff|woff2|eot|ttf)$/,
          use: "file-loader"
        },
        {
          test: /\.(less)$/,
          use: [
            'style-loader',
            { loader: 'css-loader', options: { importLoaders: 1 } },
            'less-loader'
          ]
        },
        {
          test: /unity-theme\/dist\/theme\.dev\.(css)$/,
          loader: ExtractTextPlugin.extract("css-loader?url=false!less-loader")
        },
        {
          test: /\.style$/,
          use: "style-loader!css-loader!less-loader!url?limit=10000"
        }
    ]
  },
  plugins: plugins,
  performance: {
    hints: false
  },
  node: {
      fs: "empty",
      net: "empty",
      module: "empty",
      dns: "empty",
      dgram: "empty",
      console: true
    }
};

module.exports = config;
